const {
	postNotification,
	addNormalNotificationObserver,
	removeNotificationObserver
} = require("../utils/notificationCenter");
import Dict from "../models/Dict.js";
import {
	apiPOST,
	apiGET
} from "../utils/apiRequest.js"
import {
	readStorage,
	saveStorage
} from "../utils/storageUtil.js";
import {
	dateFormat,
	stringToDate
} from "../utils/timeUtil.js";
import {
	isEmpty
} from "../utils/util.js";

/** ================= 阅读时间 ================= */
let READ_TIME_COUNT_MAP = null;
let KEY_STORAGE_READ_TIME_COUNT = "READ_TIME_COUNT";
/**
 * 读取阅读时间计时
 */
function getReadTimeCount() {
	if (!READ_TIME_COUNT_MAP) {
		READ_TIME_COUNT_MAP = readStorage(KEY_STORAGE_READ_TIME_COUNT) || {}
	}
	let date = dateFormat(new Date(), "yyyy-MM-dd");
	let count = READ_TIME_COUNT_MAP[date] || 0;
	return count;
}
/**
 * 存储阅读时间计时
 */
function setReadTimeCount(count) {
	if (!READ_TIME_COUNT_MAP) {
		READ_TIME_COUNT_MAP = readStorage(KEY_STORAGE_READ_TIME_COUNT) || {}
	}
	let date = dateFormat(new Date(), "yyyy-MM-dd");
	let result = READ_TIME_COUNT_MAP[date] || 0;
	result = result + count;
	READ_TIME_COUNT_MAP[date] = result;
	saveStorage(KEY_STORAGE_READ_TIME_COUNT, READ_TIME_COUNT_MAP);
}

let startReadTime = null; // 开始阅读时间
/**
 * 开始阅读计时
 */
function startCountReadTime() {
	startReadTime = new Date();
}
/**
 * 停止阅读计时
 */
function endCountReadTime() {
	if (!startReadTime) return;
	let endReadTime = new Date();
	let start = dateFormat(startReadTime, "yyyy-MM-dd");
	start = start.split("-");
	let end = dateFormat(endReadTime, "yyyy-MM-dd");
	end = end.split("-");
	let isSameDay = true;
	start.forEach((item, index) => {
		if (item != end[index]) {
			isSameDay = false;
		}
	})
	let count = 0;
	if (isSameDay) {
		count = endReadTime.getTime() - startReadTime.getTime();
	} else {
		let dayStart = stringToDate(end.join("-"))
		count = endReadTime.getTime() - dayStart.getTime();
	}
	startReadTime = null;
	setReadTimeCount(count);
}

/** ================= 收藏 ================= */
const KEY_NOTIFICATION_COLLECTION_CHANGE = 'COLLECTION_CHANGE';
/**
 * 收藏列表变动通知

 * @param {Object} bookId
 * @param {Object} isCollect
 */
function noticeCollectionListChange(bookId, isCollect) {
	postNotification(KEY_NOTIFICATION_COLLECTION_CHANGE, {
		bookId,
		isCollect
	});
}
/**
 * 收藏列表变动监听

 * @param {Object} fn
 * @param {Object} observer
 */
function watchCollectionChange(fn, observer) {
	addNormalNotificationObserver(KEY_NOTIFICATION_COLLECTION_CHANGE, fn, observer);
}
/**
 * 移除收藏列表变动监听

 * @param {Object} observer
 */
function removeCollectionChangeWatch(observer) {
	removeNotificationObserver(KEY_NOTIFICATION_COLLECTION_CHANGE, observer);
}

/** ================= 字典 ================= */
const DictMap = {}; // 系统字典缓存数据
const ENUM_DICT_NAME = {
	SEX: "sys_user_sex"
}

/**
 * 获取系统字典
 */
function getSystemDict(types = [], callback) {
	let result = {};
	let emptyKeys = [];
	types.map(key => {
		let value = DictMap[key];
		if (value) {
			result[key] = value;
		} else {
			emptyKeys.push(key);
		}
	})
	if (isEmpty(emptyKeys)) {
		if (typeof callback == 'function') callback(true, result);
		return;
	}
	apiPOST({
		url: `/system/dictData/getDataByDictType`,
		data: emptyKeys,
		callback: (success, data) => {
			if (success) {
				Object.keys(data).map(key => {
					DictMap[key] = new Dict(data[key])
				});
				getSystemDict(types, callback);
			} else {
				if (typeof callback == 'function') callback(false)
			}
		}
	})
}

/**
 * 收藏、取消收藏书籍

 * @param {Object} isCollected
 * @param {Object} bookId
 * @param {Object} callback
 */
function collectionBook(isCollected, bookId, callback) {
	let url = isCollected ? '/book/collect' : "/book/collectCancel";
	// url = `${url}?articleId=${bookId}`
	apiPOST({
		url,
		data: {
			articleId:bookId
		},
		callback
	})
}

/** 获取充值入口开关

 * @param {Object} callback
 */
function getOpens(callback) {
	apiGET({
		url: `/v1/article/opens`,
		callback
	})
}

/**
 * 获取VIP套餐列表
 
 * @param {Object} callback
 */
function getPackData(callback) {
	apiPOST({
		url: `/vip/getVipProducts/point`,
		callback
	})
}

/**
 * 提交反馈
 */
function getFeedback(text, callback) {
	apiPOST({
		url: `/v1/feedback/`,
		data: {
			title: '反馈',
			content: text,
			type: 1
		},
		callback
	})
}

/**
 * 获取openId

 * @param {Object} code wx code
 * @param {Object} callback
 */
function getOpenId(code, callback) {
	apiPOST({
		url: `/getOpenId`,
		data: {
			code
		},
		callback
	})
}

/**
 * 支付类型
 */
const ENUM_PAY_TYPE = {
	VIP: {
		value: 1,
		name: "购买会员"
	},
	BEAN: {
		value: 2,
		name: "购买书豆"
	}
}

/**
 * 获取开通VIP支付信息

 * @param {Object} id
 * @param {Object} amount
 * @param {Object} openId
 * @param {Object} callback
 */
function getPayInfo(id, amount, openId, callback, typeId = ENUM_PAY_TYPE.VIP.value) {
	apiPOST({
		url: `/wx/pay/createOrder`,
		data: {
			typeId,
			memberTypeId: id,
			amount: Number(amount),
			openId
		},
		callback
	})
}

function getPayInfoDy(pid) {
	apiPOST({
		url: `/vip/getVipPayParams/douyin`,
		data: {
			pid: pid
		},
		callback
	})
}

/**
 * 获取书豆套餐数据

 * @param {Object} callback
 */
function getBookBeanPackData(callback) {
	apiPOST({
		url: `//system/bookLegumesType/list`,
		callback
	})
}

/**
 * 解锁书籍

 * @param {Object} bookId
 * @param {Object} callback
 */
function buyBookWithBookBean(bookId, callback) {
	apiPOST({
		url: `/system/bookUser/create`,
		data: {
			articleId: bookId
		},
		callback
	})
}
module.exports = {
	getOpens,
	getReadTimeCount,
	startCountReadTime,
	endCountReadTime,

	noticeCollectionListChange,
	watchCollectionChange,
	removeCollectionChangeWatch,
	collectionBook,

	ENUM_DICT_NAME,
	getSystemDict,

	getBookBeanPackData,
	getPackData,
	getOpenId,
	getFeedback,
	ENUM_PAY_TYPE,
	getPayInfoDy,
	getPayInfo,

	buyBookWithBookBean,

}